/***************************************************************************
 *   Copyright (C) 2007  Antony Martin <martin.antony@yahoo.fr>            *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or         *
 *   modify it under the terms of the GNU General Public License           *
 *   as published by the Free Software Foundation; either version 2        *
 *   of the License, or (at your option) any later version.                *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA          *
 ***************************************************************************/

/* Cree le : 12 octobre 2006
   derniere modification le 09/01/2008 */

#include <SCE/SCEngine.h>

/**
 * \defgroup SCEngine
 */

/** { */


/* revise le 09/01/2008 */
/**
 * \brief Initialize SCEngine
 * This function initializes all sub modules of SCEngine.
 * \param outlog File used to log informations and errors
 * \returns SCE_ERROR if any error has occured, else SCE_OK
 */

int SCE_Init (FILE *outlog, SCEflags flags)
{
    SCE_btstart ();
    /* initialisation des outils */
    if (outlog)
        Logger_Init (outlog);
    else
    {
        fputs ("error: invalid output stream of errors", stderr);
        SCE_btend ();
        return SCE_ERROR;
    }
    if (SCE_Init_Matrix () < 0)
    {
        Logger_LogFinish ("can't initialize matrices manager");
        SCE_btend ();
        return SCE_ERROR;
    }
    if (SCE_Init_Media () < 0)
    {
        Logger_LogFinish ("can't initialize medias manager");
        SCE_btend ();
        return SCE_ERROR;
    }
    if (SCE_Init_Resource () < 0)
    {
        Logger_LogFinish ("can't initialize resources manager");
        SCE_btend ();
        return SCE_ERROR;
    }
    /***/

    /* initialisation du coeur */
    if (SCE_CInit (flags) < 0)
    {
        Logger_LogFinish ("can't initialize core!");
        SCE_btend ();
        return SCE_ERROR;
    }
    /***/

    /* initialisation des modules de l'interface */
    /* initialisation du gestionnaire de textures */
    if (SCE_Init_Texture () < 0)
    {
        SCE_Quit();
        Logger_LogFinish("can't initialize textures manager");
        SCE_btend ();
        return (SCE_ERROR);
    }
    /* initialisation du gestionnaire de shaders */
    if (SCE_Init_Shader () < 0)
    {
        SCE_Quit();
        Logger_LogFinish("can't initialize shaders manager");
        return (SCE_ERROR);
    }
    /* initialisation du gestionnaire de maillages */
    if (SCE_Init_Mesh () < 0)
    {
        SCE_Quit ();
        Logger_LogFinish ("can't initialize meshs manager");
        SCE_btend ();
        return SCE_ERROR;
    }
    /* initialisation du gestionnaire de modeles */
    if (SCE_Init_Model () < 0)
    {
        SCE_Quit ();
        Logger_LogFinish ("can't initialize models manager");
        SCE_btend ();
        return SCE_ERROR;
    }
    /***/

    SCE_btend ();
    return SCE_OK;
}

/* revise le 20/10/2007 */
/**
 * \brief Quit SCE_Engine and free memory used
 * This function frees every modules initialised by SCE_Init
 * \returns SCE_ERROR if any error has occured, else SCE_OK 
 */
void SCE_Quit (void)
{
    SCE_btstart ();
    SCE_Quit_Model ();
    SCE_Quit_Shader ();
    SCE_Quit_Texture ();
    SCE_CQuit ();
    SCE_Quit_Resource ();
    SCE_Quit_Media ();
    SCE_btend ();
}


/** } */
