/***************************************************************************
 *   Copyright (C) 2007  Antony Martin <martin.antony@yahoo.fr>            *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or         *
 *   modify it under the terms of the GNU General Public License           *
 *   as published by the Free Software Foundation; either version 2        *
 *   of the License, or (at your option) any later version.                *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA          *
 ***************************************************************************/

/* Cree le : 16/05/2007
   derniere modification : 03/02/2008 */

#include <SCE/SCEMinimal.h>

#include <SCE/interface/SCEModel.h>


int SCE_Init_Model (void)
{
    return SCE_OK;
}

void SCE_Quit_Model (void)
{
}


/* revise le 21/02/2008 */
static void SCE_Model_Init (SCE_SModel *mdl)
{
    mdl->meshs = NULL;
    mdl->lod = 0;
    mdl->mat = NULL;
    mdl->cullface = SCE_TRUE; /* backface culling active par defaut */
}


/* revise le 15/01/2008 */
SCE_SModel* SCE_Model_Create (void)
{
    SCE_SModel *mdl = NULL;

    SCE_btstart ();
    mdl = SCE_malloc (sizeof *mdl);
    if (!mdl)
    {
        Logger_LogSrc ();
        SCE_btend ();
        return NULL;
    }

    SCE_Model_Init (mdl);

    SCE_btend ();
    return mdl;
}

/* revise le 15/01/2008 */
void SCE_Model_Delete (SCE_SModel *mdl)
{
    SCE_btstart ();
    if (mdl)
    {
        SCE_Mesh_DeleteList (mdl->meshs);
        SCE_Material_Delete (mdl->mat);
        SCE_free (mdl);
    }
    SCE_btend ();
}


/* ajoute le 14/06/2007 */
/* revise le 15/01/2008 */
void SCE_Model_Bind (SCE_SModel *mdl)
{
    if (mdl)
    {
        SCE_Mesh_Use (mdl->meshs[mdl->lod]);
        SCE_Material_Use (mdl->mat);
    }
    else
    {
        SCE_Material_Use (NULL);
        SCE_Mesh_Use (NULL);
    }
}

/* ajoute le 14/06/2007 */
/* revise le 15/01/2008 */
void SCE_Model_BindMesh (SCE_SModel *mdl)
{
    if (mdl)
        SCE_Mesh_Use (mdl->meshs[mdl->lod]);
    else
        SCE_Mesh_Use (NULL);
}

/* ajoute le 14/06/2007 */
void SCE_Model_BindMaterial (SCE_SModel *mdl)
{
    if (mdl)
        SCE_Material_Use (mdl->mat);
    else
        SCE_Material_Use (NULL);
}

/* ajoute le 14/06/2007 */
void SCE_Model_Draw (void)
{
    SCE_Mesh_Draw ();
}

/* revise le 15/01/2008 */
void SCE_Model_Render (SCE_SModel *mdl)
{
    SCE_Model_Bind (mdl);
    SCE_Model_Draw ();
    SCE_Model_Bind (NULL);
}


/* ajoute le 25/06/2007 */
/* revise le 17/02/2008 */
int SCE_Model_LoadMesh (SCE_SModel *mdl, const char *fname)
{
    SCE_SMesh **meshs = NULL;
    
    SCE_btstart ();
    meshs = SCE_Mesh_Load (fname, NULL);
    if (!meshs)
    {
        Logger_LogSrc ();
        SCE_btend ();
        return SCE_ERROR;
    }
    SCE_Mesh_DeleteList (mdl->meshs);
    mdl->meshs = meshs;
    SCE_btend ();
    return SCE_OK;
}
